_G.BagContour = _G.BagContour or {}
BagContour._path = ModPath
BagContour._data_path = SavePath .. "bag_contour.txt"
BagContour._ovrd_path = SavePath .. "bag_contour_overrides.txt"
BagContour._bags = {}
BagContour._ptexts = {}
BagContour._overrides = {}
BagContour.settings = {}

function BagContour:Reset()
	self.settings = {
		LR = 2.00,
		LG = 2.55,
		LB = 0,
		HR = 2.55,
		HG = 0,
		HB = 0.40,
		BR = 0.20,
		BG = 0.80,
		BB = 1
	}
end

function BagContour:UpdateOverrides()
	self._overrides = {}

	local file = io.open(self._ovrd_path, "r")
	if file then
		for _, line in ipairs(string.split(file:read("*all") or "", "\n")) do
			local r, g, b, carry_id = line:match('^RGB%((%d+),(%d+),(%d+)%)[ 	]+([^ 	]*)$')
			if carry_id then
				self._overrides[carry_id] = Vector3(r / 255, g / 255, b / 255)
			end
		end
		file:close()
	end

	file = io.open(self._ovrd_path, "w")
	if file then
		local out = {}
		for carry_id, data in pairs(tweak_data.carry) do
			if data.type then
				local v = self._overrides[carry_id]
				local info = "" -- data.name_id and (" (" .. managers.localization:text(data.name_id) .. ")") or ""
				if v then
					table.insert(out, string.format("RGB(%i,%i,%i)	%s%s", v[1] * 255, v[2] * 255, v[3] * 255, carry_id, info))
				else
					table.insert(out, "weight		" .. carry_id .. info)
				end
			end
		end
		table.sort(out, function(a, b) return a < b end)
		file:write(table.concat(out, "\n"))
		file:close()
	end
end

function BagContour:Load()
	local file = io.open(self._data_path, "r")
	if file then
		for k, v in pairs(json.decode(file:read("*all"))) do
			self.settings[k] = v
		end
		file:close()
	else
		self:Reset()
	end
end

function BagContour:Save()
	local file = io.open(self._data_path, "w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

function BagContour:RenewLabelExamples()
	local carry_by_msm = {}
	for carry_id, data in pairs(tweak_data.carry) do
		if data.type and carry_id ~= "person" then
			local msm = tweak_data.carry.types[data.type].move_speed_modifier
			carry_by_msm[msm] = carry_by_msm[msm] or {}
			table.insert(carry_by_msm[msm], carry_id)
		end
	end

	local sorted_msms = {}
	for msm, _ in pairs(self.move_speed_modifiers) do
		table.insert(sorted_msms, msm)
	end
	table.sort(sorted_msms, function(a, b) return a > b end)

	self.examples = {}
	self._labels = {}
	for i, msm in ipairs(sorted_msms) do
		local rnd = carry_by_msm[msm][math.random(#carry_by_msm[msm])]
		table.insert(self.examples, rnd)
		self._labels[i] = tweak_data.carry[rnd].name_id
	end
end

function BagContour:SetContourColors()
	self.move_speed_modifiers = {}
	local msm_min = 10
	local msm_max = 0
	for carry_id, data in pairs(tweak_data.carry.types) do
		if not data.type and data.move_speed_modifier and carry_id ~= "person" and not self.move_speed_modifiers[data.move_speed_modifier] then
			self.move_speed_modifiers[data.move_speed_modifier] = 1
			msm_min = math.min(msm_min, data.move_speed_modifier)
			msm_max = math.max(msm_max, data.move_speed_modifier)
		end
	end

	local color_light = { self.settings.LR / 2.55, self.settings.LG / 2.55, self.settings.LB / 2.55 }
	local color_heavy = { self.settings.HR / 2.55, self.settings.HG / 2.55, self.settings.HB / 2.55 }
	local msm_amplitude = msm_max - msm_min
	for msm, _ in pairs(self.move_speed_modifiers) do
		local ratio = (msm_max - msm) / msm_amplitude
		self.move_speed_modifiers[msm] = Vector3(
			color_light[1] + (color_heavy[1] - color_light[1]) * ratio,
			color_light[2] + (color_heavy[2] - color_light[2]) * ratio,
			color_light[3] + (color_heavy[3] - color_light[3]) * ratio
		)
	end

	for carry_id, data in pairs(tweak_data.carry) do
		if data.type then
			tweak_data.contour.interactable[carry_id] = self._overrides[carry_id] or self.move_speed_modifiers[tweak_data.carry.types[data.type].move_speed_modifier]
		end
	end

	if not self.examples then
		self:RenewLabelExamples()
	end

	tweak_data.contour.interactable.person = Vector3(self.settings.BR / 2.55, self.settings.BG / 2.55, self.settings.BB / 2.55)
end

function BagContour:CanHideBodyBagContour()
	if Global.game_settings.level_id == "mad" then
		return false
	end

	return managers.groupai:state()._police_called
end

function BagContour:SetBagContour(unit, contour_id)
	local ids_contour_color = Idstring("contour_color")
	local ids_contour_opacity = Idstring("contour_opacity")
	for _, m in ipairs(unit:get_objects_by_type(Idstring("material"))) do
		if m:variable_exists(ids_contour_color) then
			m:set_variable(ids_contour_color, tweak_data.contour.interactable[contour_id])
			m:set_variable(ids_contour_opacity, 1)
		end
	end
end

function BagContour:SetAllBagsContour()
	for i, carry_id in ipairs(self.examples) do
		self:SetBagContour(self._bags[i], carry_id)
	end
	self:SetBagContour(self._bodybag, "person")
end

function BagContour:SetAllLabelsColor()
	for i, carry_id in ipairs(self.examples) do
		local c = tweak_data.contour.interactable[carry_id]
		self._ptexts[i]:set_color(Color(c.x, c.y, c.z))
	end
end

function BagContour:SpawnBag(carry_id, h)
	local unit_name = "units/payday2/pickups/gen_pku_lootbag/gen_pku_lootbag"
	local unit = World:spawn_unit(Idstring(unit_name), Vector3(45, 300, 175 - h * 50), Rotation(math.random() * 20 - 10, math.random() * 20 - 10, math.random() * 4 - 2))
	self._bags[h] = unit
	for i = 0, unit:num_bodies() - 1 do
		unit:body(i):set_keyframed()
	end
end

function BagContour:SpawnBodybag()
	local unit_name = "units/payday2/pickups/gen_pku_bodybag/gen_pku_bodybag"
	local unit = World:spawn_unit(Idstring(unit_name), Vector3(150, 250, -40), Rotation(0, 0, 0))
	self._bodybag = unit
	for i = 0, unit:num_bodies() - 1 do
		unit:body(i):set_keyframed()
	end
end

function BagContour:UpdateColors()
	self:SetContourColors()
	if not managers.hud then
		self:SetAllLabelsColor()
		self:SetAllBagsContour()
	end
end

function BagContour:CreateWeightLabels()
	self._panel = managers.menu_component._ws:panel():panel()

	for i, txt in ipairs(self._labels) do
		self._ptexts[i] = self._panel:text({
			text = managers.localization:text(txt),
			align = "center",
			vertical = "top",
			font_size = tweak_data.menu.pd2_small_font_size - 4,
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.screen_colors.text,
		})
		local _, _, w, h = self._ptexts[i]:text_rect()
		self._ptexts[i]:set_size(w, h)
		local pos = managers.menu_component._ws:world_to_screen(managers.menu_scene._camera_object, self._bags[i]:position())
		self._ptexts[i]:set_center(pos.x - 30, pos.y)
	end
end

function BagContour:RemoveWeightLabels()
	if self._panel and alive(self._panel) then
		self._panel:clear()
		self._panel:parent():remove(self._panel)
	end
	self._ptexts = {}
	self._panel = nil
end

Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_BagContour", function(loc)
	if _G.PD2KR then
		loc:load_localization_file(BagContour._path .. "loc/korean.txt")
	else
		for _, filename in pairs(file.GetFiles(BagContour._path .. "loc/")) do
			local str = filename:match('^(.*).txt$')
			if str and Idstring(str) and Idstring(str):key() == SystemInfo:language():key() then
				loc:load_localization_file(BagContour._path .. "loc/" .. filename)
				break
			end
		end
	end
	
	loc:load_localization_file(BagContour._path .. "loc/english.txt", false)
end)

Hooks:Add("MenuManagerInitialize", "MenuManagerInitialize_BagContour", function(menu_manager)
	MenuCallbackHandler.BagContourChangedFocus = function(node, focus)
		if managers.menu:active_menu().name ~= "menu_main" then
			return
		end
		
		if focus then
			for i, carry_id in ipairs(BagContour.examples) do
				BagContour:SpawnBag(carry_id, i)
			end
			BagContour:SpawnBodybag()
			BagContour:SetAllBagsContour()
			BagContour:CreateWeightLabels()
			BagContour:SetAllLabelsColor()

		else
			for i, carry_id in ipairs(BagContour.examples) do
				if BagContour._bags[i] and alive(BagContour._bags[i]) then
					World:delete_unit(BagContour._bags[i])
				end
				BagContour._bags[i] = nil
			end
			if BagContour._bodybag and alive(BagContour._bodybag) then
				World:delete_unit(BagContour._bodybag)
			end
			BagContour._bodybag = nil
			BagContour:RemoveWeightLabels()
			BagContour._panel = nil
			
		end
	end

	MenuCallbackHandler.BagContourSetRedLight = function(this, item)
		if BagContour.settings.LR ~= tonumber(item:value()) then
			BagContour.settings.LR = tonumber(item:value())
			BagContour:UpdateColors()
		end
	end

	MenuCallbackHandler.BagContourSetGreenLight = function(this, item)
		if BagContour.settings.LG ~= tonumber(item:value()) then
			BagContour.settings.LG = tonumber(item:value())
			BagContour:UpdateColors()
		end
	end

	MenuCallbackHandler.BagContourSetBlueLight = function(this, item)
		if BagContour.settings.LB ~= tonumber(item:value()) then
			BagContour.settings.LB = tonumber(item:value())
			BagContour:UpdateColors()
		end
	end

	MenuCallbackHandler.BagContourSetRedHeavy = function(this, item)
		if BagContour.settings.HR ~= tonumber(item:value()) then
			BagContour.settings.HR = tonumber(item:value())
			BagContour:UpdateColors()
		end
	end

	MenuCallbackHandler.BagContourSetGreenHeavy = function(this, item)
		if BagContour.settings.HG ~= tonumber(item:value()) then
			BagContour.settings.HG = tonumber(item:value())
			BagContour:UpdateColors()
		end
	end

	MenuCallbackHandler.BagContourSetBlueHeavy = function(this, item)
		if BagContour.settings.HB ~= tonumber(item:value()) then
			BagContour.settings.HB = tonumber(item:value())
			BagContour:UpdateColors()
		end
	end

	MenuCallbackHandler.BagContourSetRedBody = function(this, item)
		if BagContour.settings.BR ~= tonumber(item:value()) then
			BagContour.settings.BR = tonumber(item:value())
			BagContour:UpdateColors()
		end
	end

	MenuCallbackHandler.BagContourSetGreenBody = function(this, item)
		if BagContour.settings.BG ~= tonumber(item:value()) then
			BagContour.settings.BG = tonumber(item:value())
			BagContour:UpdateColors()
		end
	end

	MenuCallbackHandler.BagContourSetBlueBody = function(this, item)
		if BagContour.settings.BB ~= tonumber(item:value()) then
			BagContour.settings.BB = tonumber(item:value())
			BagContour:UpdateColors()
		end
	end

	MenuCallbackHandler.BagContourReset = function(this, item)
		BagContour:Reset()
		MenuHelper:ResetItemsToDefaultValue(item, {["bc_slider_colour_light_r"] = true}, BagContour.settings.LR)
		MenuHelper:ResetItemsToDefaultValue(item, {["bc_slider_colour_light_g"] = true}, BagContour.settings.LG)
		MenuHelper:ResetItemsToDefaultValue(item, {["bc_slider_colour_light_b"] = true}, BagContour.settings.LB)
		MenuHelper:ResetItemsToDefaultValue(item, {["bc_slider_colour_heavy_r"] = true}, BagContour.settings.HR)
		MenuHelper:ResetItemsToDefaultValue(item, {["bc_slider_colour_heavy_g"] = true}, BagContour.settings.HG)
		MenuHelper:ResetItemsToDefaultValue(item, {["bc_slider_colour_heavy_b"] = true}, BagContour.settings.HB)
		MenuHelper:ResetItemsToDefaultValue(item, {["bc_slider_colour_body_r"] = true}, BagContour.settings.BR)
		MenuHelper:ResetItemsToDefaultValue(item, {["bc_slider_colour_body_g"] = true}, BagContour.settings.BG)
		MenuHelper:ResetItemsToDefaultValue(item, {["bc_slider_colour_body_b"] = true}, BagContour.settings.BB)
		BagContour:UpdateColors()
	end

	MenuCallbackHandler.BagContourSave = function(this, item)
		BagContour:Save()
	end

	BagContour:Load()
	BagContour:UpdateOverrides()
	BagContour:SetContourColors()
	MenuHelper:LoadFromJsonFile(BagContour._path .. "menu/options.txt", BagContour, BagContour.settings)
end)
